from canokit.management import CAPABILITY
from ... import condition
import pytest


@pytest.fixture(autouse=True)
@condition.capability(CAPABILITY.PIV)
def ensure_piv(ckman_cli):
    ckman_cli("piv", "reset", "-f")



from canokit.core import TRANSPORT
from ckman._cli.__main__ import cli, _DefaultFormatter
from ckman._cli.aliases import apply_aliases
from ckman._cli.util import CliFail
from click.testing import CliRunner
from functools import partial
import logging
import pytest


@pytest.fixture(scope="module")
def ckman_cli(device, info):
    if device.transport == TRANSPORT.NFC:
        return partial(_ckman_cli, "--reader", device.reader.name)
    elif info.serial is not None:
        return partial(_ckman_cli, "--device", info.serial)
    else:
        return _ckman_cli


def _ckman_cli(*argv, **kwargs):
    handler = logging.StreamHandler()
    handler.setLevel(logging.WARNING)
    handler.setFormatter(_DefaultFormatter())
    logging.getLogger().addHandler(handler)

    argv = apply_aliases(["ckman"] + [str(a) for a in argv])
    runner = CliRunner(mix_stderr=False)
    result = runner.invoke(cli, argv[1:], obj={}, **kwargs)
    if result.exit_code != 0:
        if isinstance(result.exception, CliFail):
            raise SystemExit()
        raise result.exception
    return result



from .util import (
    old_new_new,
    DEFAULT_PIN,
    DEFAULT_MANAGEMENT_KEY,
    NON_DEFAULT_MANAGEMENT_KEY,
)
import re
import pytest


class TestManagementKey:
    def test_change_management_key_force_fails_without_generate(self, ckman_cli):
        with pytest.raises(SystemExit):
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                "-P",
                DEFAULT_PIN,
                "-m",
                DEFAULT_MANAGEMENT_KEY,
                "-f",
            )

    def test_change_management_key_protect_random(self, ckman_cli):
        ckman_cli(
            "piv",
            "access",
            "change-management-key",
            "-p",
            "-P",
            DEFAULT_PIN,
            "-m",
            DEFAULT_MANAGEMENT_KEY,
        )
        output = ckman_cli("piv", "info").output
        assert "Management key is stored on the YubiKey, protected by PIN" in output

        with pytest.raises(SystemExit):
            # Should fail - wrong current key
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                "-p",
                "-P",
                DEFAULT_PIN,
                "-m",
                DEFAULT_MANAGEMENT_KEY,
            )

        # Should succeed - PIN as key
        ckman_cli("piv", "access", "change-management-key", "-p", "-P", DEFAULT_PIN)

    def test_change_management_key_protect_prompt(self, ckman_cli):
        ckman_cli(
            "piv",
            "access",
            "change-management-key",
            "-p",
            "-P",
            DEFAULT_PIN,
            input=DEFAULT_MANAGEMENT_KEY,
        )
        output = ckman_cli("piv", "info").output
        assert "Management key is stored on the YubiKey, protected by PIN" in output

        with pytest.raises(SystemExit):
            # Should fail - wrong current key
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                "-p",
                "-P",
                DEFAULT_PIN,
                "-m",
                DEFAULT_MANAGEMENT_KEY,
            )

        # Should succeed - PIN as key
        ckman_cli("piv", "access", "change-management-key", "-p", "-P", DEFAULT_PIN)

    def test_change_management_key_no_protect_generate(self, ckman_cli):
        output = ckman_cli(
            "piv", "access", "change-management-key", "-m", DEFAULT_MANAGEMENT_KEY, "-g"
        ).output

        assert re.match(
            r"^Generated management key: [a-f0-9]{48}$", output, re.MULTILINE
        )

        output = ckman_cli("piv", "info").output
        assert "Management key is stored on the YubiKey" not in output

    def test_change_management_key_no_protect_arg(self, ckman_cli):
        output = ckman_cli(
            "piv",
            "access",
            "change-management-key",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            "-n",
            NON_DEFAULT_MANAGEMENT_KEY,
        ).output
        assert "" == output
        output = ckman_cli("piv", "info").output
        assert "Management key is stored on the YubiKey" not in output

        with pytest.raises(SystemExit):
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                "-m",
                DEFAULT_MANAGEMENT_KEY,
                "-n",
                NON_DEFAULT_MANAGEMENT_KEY,
            )

        output = ckman_cli(
            "piv",
            "access",
            "change-management-key",
            "-m",
            NON_DEFAULT_MANAGEMENT_KEY,
            "-n",
            DEFAULT_MANAGEMENT_KEY,
        ).output
        assert "" == output

    def test_change_management_key_no_protect_arg_bad_length(self, ckman_cli):
        with pytest.raises(SystemExit):
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                "-m",
                DEFAULT_MANAGEMENT_KEY,
                "-n",
                "10020304050607080102030405060708",
            )

    def test_change_management_key_no_protect_prompt(self, ckman_cli):
        output = ckman_cli(
            "piv",
            "access",
            "change-management-key",
            input=old_new_new(DEFAULT_MANAGEMENT_KEY, NON_DEFAULT_MANAGEMENT_KEY),
        ).output
        assert "Generated" not in output
        output = ckman_cli("piv", "info").output
        assert "Management key is stored on the YubiKey" not in output

        with pytest.raises(SystemExit):
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                input=old_new_new(DEFAULT_MANAGEMENT_KEY, NON_DEFAULT_MANAGEMENT_KEY),
            )

        ckman_cli(
            "piv",
            "access",
            "change-management-key",
            input=old_new_new(NON_DEFAULT_MANAGEMENT_KEY, DEFAULT_MANAGEMENT_KEY),
        )
        assert "Generated" not in output

    def test_change_management_key_new_key_conflicts_with_generate(self, ckman_cli):
        with pytest.raises(SystemExit):
            ckman_cli(
                "piv",
                "access",
                "change-management-key",
                "-m",
                DEFAULT_MANAGEMENT_KEY,
                "-n",
                NON_DEFAULT_MANAGEMENT_KEY,
                "-g",
            )



def pytest_addoption(parser):
    parser.addoption("--device", action="store", type=int)
    parser.addoption("--reader", action="store")
    parser.addoption("--no-serial", action="store_true")
    parser.addoption("--use-version", action="store")



# -*- coding: utf-8 -*-

from ckman.oath import STEAM_CHAR_TABLE
from canokit.management import CAPABILITY
from .. import condition
from base64 import b32encode
import pytest


URI_HOTP_EXAMPLE = (
    "otpauth://hotp/Example:demo@example.com?"
    "secret=JBSWY3DPK5XXE3DEJ5TE6QKUJA======&issuer=Example&counter=1"
)

URI_TOTP_EXAMPLE = (
    "otpauth://totp/ACME%20Co:john.doe@email.com?"
    "secret=HXDMVJECJJWSRB3HWIZR4IFUGFTMXBOZ&issuer=ACME%20Co"
    "&algorithm=SHA1&digits=6&period=30"
)

URI_TOTP_EXAMPLE_B = (
    "otpauth://totp/ACME%20Co:john.doe.b@email.com?"
    "secret=HXDMVJECJJWSRB3HWIZR4IFUGFTMXBOZ&issuer=ACME%20Co"
    "&algorithm=SHA1&digits=6&period=30"
)

URI_TOTP_EXAMPLE_EXTRA_PARAMETER = (
    "otpauth://totp/ACME%20Co:john.doe.extra@email.com?"
    "secret=HXDMVJECJJWSRB3HWIZR4IFUGFTMXBOZ&issuer=ACME%20Co"
    "&algorithm=SHA1&digits=6&period=30&skid=JKS3424d"
)

PASSWORD = "aaaa"


@pytest.fixture(autouse=True)
@condition.capability(CAPABILITY.OATH)
def preconditions(ckman_cli):
    ckman_cli("oath", "reset", "-f")


class TestOATH:
    def test_oath_info(self, ckman_cli):
        output = ckman_cli("oath", "info").output
        assert "version:" in output

    @condition.yk4_fips(False)
    def test_info_does_not_indicate_fips_mode_for_non_fips_key(self, ckman_cli):
        info = ckman_cli("oath", "info").output
        assert "FIPS:" not in info

    def test_oath_add_credential(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "test-name", "abba")
        creds = ckman_cli("oath", "accounts", "list").output
        assert "test-name" in creds

    def test_oath_add_credential_prompt(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "test-name-2", input="abba")
        creds = ckman_cli("oath", "accounts", "list").output
        assert "test-name-2" in creds

    def test_oath_add_credential_with_space(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "test-name-space", "ab ba")
        creds = ckman_cli("oath", "accounts", "list").output
        assert "test-name-space" in creds

    def test_oath_hidden_cred(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "_hidden:name", "abba")
        creds = ckman_cli("oath", "accounts", "code").output
        assert "_hidden:name" not in creds
        creds = ckman_cli("oath", "accounts", "code", "-H").output
        assert "_hidden:name" in creds

    def test_oath_add_uri_hotp(self, ckman_cli):
        ckman_cli("oath", "accounts", "uri", URI_HOTP_EXAMPLE)
        creds = ckman_cli("oath", "accounts", "list").output
        assert "Example:demo" in creds

    def test_oath_add_uri_totp(self, ckman_cli):
        ckman_cli("oath", "accounts", "uri", URI_TOTP_EXAMPLE)
        creds = ckman_cli("oath", "accounts", "list").output
        assert "john.doe" in creds

    def test_oath_add_uri_totp_extra_parameter(self, ckman_cli):
        ckman_cli("oath", "accounts", "uri", URI_TOTP_EXAMPLE_EXTRA_PARAMETER)
        creds = ckman_cli("oath", "accounts", "list").output
        assert "john.doe.extra" in creds

    def test_oath_add_uri_totp_prompt(self, ckman_cli):
        ckman_cli("oath", "accounts", "uri", input=URI_TOTP_EXAMPLE_B)
        creds = ckman_cli("oath", "accounts", "list").output
        assert "john.doe" in creds

    def test_oath_code(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "test-name2", "abba")
        creds = ckman_cli("oath", "accounts", "code").output
        assert "test-name2" in creds

    def test_oath_code_query_single(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "query-me", "abba")
        creds = ckman_cli("oath", "accounts", "code", "query-me").output
        assert "query-me" in creds

    def test_oath_code_query_multiple(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "foo", "abba")
        ckman_cli("oath", "accounts", "add", "query-me", "abba")
        ckman_cli("oath", "accounts", "add", "bar", "abba")
        lines = (
            ckman_cli("oath", "accounts", "code", "query").output.strip().splitlines()
        )
        assert len(lines) == 1
        assert "query-me" in lines[0]

    def test_oath_reset(self, ckman_cli):
        output = ckman_cli("oath", "reset", "-f").output
        assert "Success! All OATH accounts have been deleted from the YubiKey" in output

    def test_oath_hotp_vectors_6(self, ckman_cli):
        ckman_cli(
            "oath",
            "accounts",
            "add",
            "-o",
            "HOTP",
            "testvector",
            b32encode(b"12345678901234567890").decode(),
        )
        for code in ["755224", "287082", "359152", "969429", "338314"]:
            words = ckman_cli("oath", "accounts", "code", "testvector").output.split()
            assert code in words

    def test_oath_hotp_vectors_8(self, ckman_cli):
        ckman_cli(
            "oath",
            "accounts",
            "add",
            "-o",
            "HOTP",
            "-d",
            "8",
            "testvector8",
            b32encode(b"12345678901234567890").decode(),
        )
        for code in ["84755224", "94287082", "37359152", "26969429", "40338314"]:
            words = ckman_cli("oath", "accounts", "code", "testvector8").output.split()
            assert code in words

    def test_oath_hotp_code(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "-o", "HOTP", "hotp-cred", "abba")
        words = ckman_cli("oath", "accounts", "code", "hotp-cred").output.split()
        assert "659165" in words

    def test_oath_hotp_code_single(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "-o", "HOTP", "hotp-cred", "abba")
        words = ckman_cli(
            "oath", "accounts", "code", "hotp-cred", "--single"
        ).output.split()
        assert "659165" in words

    def test_oath_totp_steam_code(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "Steam:steam-cred", "abba")
        cred = ckman_cli("oath", "accounts", "code", "steam-cred").output.strip()
        code = cred.split()[-1]
        assert 5 == len(code), f"cred wrong length: {code!r}"
        assert all(
            c in STEAM_CHAR_TABLE for c in code
        ), f"{code!r} contains non-steam characters"

    def test_oath_totp_steam_code_single(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "Steam:steam-cred", "abba")
        code = ckman_cli("oath", "accounts", "code", "-s", "steam-cred").output.strip()
        assert 5 == len(code), f"cred wrong length: {code!r}"
        assert all(
            c in STEAM_CHAR_TABLE for c in code
        ), f"{code!r} contains non-steam characters"

    def test_oath_code_output_no_touch(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "TOTP:normal", "aaaa")
        ckman_cli("oath", "accounts", "add", "Steam:normal", "aaba")
        ckman_cli("oath", "accounts", "add", "-o", "HOTP", "HOTP:normal", "abaa")

        lines = ckman_cli("oath", "accounts", "code").output.strip().splitlines()
        entries = {line.split()[0]: line for line in lines}
        assert "HOTP Account" in entries["HOTP:normal"]

        code = entries["Steam:normal"].split()[-1]
        assert 5 == len(code), f"cred wrong length: {code!r}"
        assert all(
            c in STEAM_CHAR_TABLE for c in code
        ), f"{code!r} contains non-steam characters"

        code = entries["TOTP:normal"].split()[-1]
        assert 6 == len(code)
        int(code)

    @condition.min_version(4)
    def test_oath_code_output(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "TOTP:normal", "aaaa")
        ckman_cli("oath", "accounts", "add", "--touch", "TOTP:touch", "aaab")
        ckman_cli("oath", "accounts", "add", "Steam:normal", "aaba")
        ckman_cli("oath", "accounts", "add", "--touch", "Steam:touch", "aabb")
        ckman_cli("oath", "accounts", "add", "-o", "HOTP", "HOTP:normal", "abaa")

        lines = ckman_cli("oath", "accounts", "code").output.strip().splitlines()
        entries = {line.split()[0]: line for line in lines}
        assert "Requires Touch" in entries["TOTP:touch"]
        assert "Requires Touch" in entries["Steam:touch"]
        assert "HOTP Account" in entries["HOTP:normal"]

        code = entries["Steam:normal"].split()[-1]
        assert 5 == len(code), f"cred wrong length: {code!r}"
        assert all(
            c in STEAM_CHAR_TABLE for c in code
        ), f"{code!r} contains non-steam characters"

        code = entries["TOTP:normal"].split()[-1]
        assert 6 == len(code)
        int(code)

    @condition.min_version(4)
    def test_oath_totp_steam_touch_not_in_code_output(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "--touch", "Steam:steam-cred", "abba")
        ckman_cli("oath", "accounts", "add", "TOTP:totp-cred", "abba")
        lines = ckman_cli("oath", "accounts", "code").output.strip().splitlines()
        assert "Requires Touch" in lines[0]

    def test_oath_delete(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "delete-me", "abba")
        ckman_cli("oath", "accounts", "delete", "delete-me", "-f")
        assert "delete-me", ckman_cli("oath", "accounts" not in "list")

    def test_oath_unicode(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "😃", "abba")
        ckman_cli("oath", "accounts", "code")
        ckman_cli("oath", "accounts", "list")
        ckman_cli("oath", "accounts", "delete", "😃", "-f")

    @condition.yk4_fips(False)
    @condition.min_version(4, 3, 1)
    def test_oath_sha512(self, ckman_cli):
        ckman_cli("oath", "accounts", "add", "abba", "abba", "--algorithm", "SHA512")
        ckman_cli("oath", "accounts", "delete", "abba", "-f")

    # NEO credential capacity may vary based on configuration
    @condition.min_version(4)
    def test_add_max_creds(self, ckman_cli, version):
        n_creds = 32 if version < (5, 7, 0) else 64
        for i in range(n_creds):
            ckman_cli("oath", "accounts", "add", "test" + str(i), "abba")
            output = ckman_cli("oath", "accounts", "list").output
            lines = output.strip().split("\n")
            assert len(lines) == i + 1

        with pytest.raises(SystemExit):
            ckman_cli("oath", "accounts", "add", "testx", "abba")

    @condition.min_version(5, 3, 1)
    def test_rename(self, ckman_cli):
        ckman_cli("oath", "accounts", "uri", URI_TOTP_EXAMPLE)
        ckman_cli(
            "oath", "accounts", "rename", "john.doe", "Example:user@example.com", "-f"
        )

        creds = ckman_cli("oath", "accounts", "list").output
        assert "john.doe" not in creds
        assert "Example:user@example.com" in creds


class TestOathFips:
    @pytest.fixture(autouse=True)
    @condition.yk4_fips(True)
    def check_fips(self):
        pass

    def test_no_fips_mode_without_password(self, ckman_cli):
        output = ckman_cli("oath", "info").output
        assert "FIPS Approved Mode: No" in output

    def test_fips_mode_with_password(self, ckman_cli):
        ckman_cli("oath", "access", "change", "-n", PASSWORD)
        output = ckman_cli("oath", "info").output
        assert "FIPS Approved Mode: Yes" in output

    def test_sha512_not_supported(self, ckman_cli):
        with pytest.raises(SystemExit):
            ckman_cli(
                "oath", "accounts", "add", "abba", "abba", "--algorithm", "SHA512"
            )



#  vim: set fileencoding=utf-8 :

from canokit.oath import (
    CredentialData,
    OATH_TYPE,
    HASH_ALGORITHM,
    _derive_key,
    _parse_cred_id,
    _format_cred_id,
)
import unittest


class TestOathFunctions(unittest.TestCase):
    def test_credential_parse_period_and_issuer_and_name(self):
        issuer, name, period = _parse_cred_id(b"20/Issuer:name", OATH_TYPE.TOTP)
        self.assertEqual(20, period)
        self.assertEqual("Issuer", issuer)
        self.assertEqual("name", name)

    def test_credential_parse_weird_issuer_and_name(self):
        issuer, name, period = _parse_cred_id(b"weird/Issuer:name", OATH_TYPE.TOTP)
        self.assertEqual(30, period)
        self.assertEqual("weird/Issuer", issuer)
        self.assertEqual("name", name)

    def test_credential_parse_issuer_and_name(self):
        issuer, name, period = _parse_cred_id(b"Issuer:name", OATH_TYPE.TOTP)
        self.assertEqual(30, period)
        self.assertEqual("Issuer", issuer)
        self.assertEqual("name", name)

    def test_credential_parse_period_and_name(self):
        issuer, name, period = _parse_cred_id(b"20/name", OATH_TYPE.TOTP)
        self.assertEqual(20, period)
        self.assertIsNone(issuer)
        self.assertEqual("name", name)

    def test_credential_parse_only_name(self):
        issuer, name, period = _parse_cred_id(b"name", OATH_TYPE.TOTP)
        self.assertEqual(30, period)
        self.assertIsNone(issuer)
        self.assertEqual("name", name)

    def test_credential_data_make_key(self):
        self.assertEqual(b"name", _format_cred_id(None, "name", OATH_TYPE.TOTP))
        self.assertEqual(
            b"Issuer:name", _format_cred_id("Issuer", "name", OATH_TYPE.TOTP)
        )
        self.assertEqual(
            b"20/Issuer:name",
            _format_cred_id("Issuer", "name", OATH_TYPE.TOTP, period=20),
        )
        self.assertEqual(
            b"Issuer:name", _format_cred_id("Issuer", "name", OATH_TYPE.TOTP, period=30)
        )
        self.assertEqual(
            b"20/name", _format_cred_id(None, "name", OATH_TYPE.TOTP, period=20)
        )

    def test_derive_key(self):
        self.assertEqual(
            b"\xb0}\xa1\xe7\xde\x87\xf8\x9a\x87\xa2\xb5\x98\xea\xa2\x18\x8c",
            _derive_key(b"\0\0\0\0\0\0\0\0", "foobar"),
        )
        self.assertEqual(
            b"\xda\x81\x8ek,\xf0\xa2\xd0\xbf\x19\xb3\xdd\xd3K\x83\xf5",
            _derive_key(b"12345678", "Hallå världen!"),
        )
        self.assertEqual(
            b"\xf3\xdf\xa7\x81T\xc8\x102\x99E\xfb\xc4\xb55\xe57",
            _derive_key(b"saltsalt", "Ťᶒśƫ ᵽĥřӓşḛ"),
        )

    def test_parse_uri_issuer(self):
        no_issuer = CredentialData.parse_uri("otpauth://totp/account" "?secret=abba")
        self.assertIsNone(no_issuer.issuer)

        from_param = CredentialData.parse_uri(
            "otpauth://totp/account" "?secret=abba&issuer=Test"
        )
        self.assertEqual("Test", from_param.issuer)

        from_name = CredentialData.parse_uri(
            "otpauth://totp/Test:account" "?secret=abba"
        )
        self.assertEqual("Test", from_name.issuer)

        with_both = CredentialData.parse_uri(
            "otpauth://totp/TestA:account" "?secret=abba&issuer=TestB"
        )
        self.assertEqual("TestB", with_both.issuer)

    def test_parse_uri(self):
        data = CredentialData.parse_uri(
            "otpauth://totp/Issuer:account"
            "?secret=abba&issuer=Issuer"
            "&algorithm=SHA256&digits=7"
            "&period=20&counter=5"
        )
        self.assertEqual(b"\0B", data.secret)
        self.assertEqual("Issuer", data.issuer)
        self.assertEqual("account", data.name)
        self.assertEqual(OATH_TYPE.TOTP, data.oath_type)
        self.assertEqual(HASH_ALGORITHM.SHA256, data.hash_algorithm)
        self.assertEqual(7, data.digits)
        self.assertEqual(20, data.period)
        self.assertEqual(5, data.counter)



# -*- coding: utf-8 -*-
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ec

from canokit.management import CAPABILITY
from canokit.hsmauth import (
    TAG_LABEL,
    TAG_CONTEXT,
    TAG_CREDENTIAL_PASSWORD,
    INS_CALCULATE,
    _parse_label,
    _parse_credential_password,
)
from canokit.core import Tlv
from .. import condition

import pytest
import re
import os
import tempfile
import struct

DEFAULT_MANAGEMENT_KEY = "00000000000000000000000000000000"
NON_DEFAULT_MANAGEMENT_KEY = "11111111111111111111111111111111"


def generate_pem_eccp256_keypair():
    pk = ec.generate_private_key(ec.SECP256R1(), default_backend())
    return (
        pk.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        ),
        pk.public_key().public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        ),
    )


@pytest.fixture()
def eccp256_keypair():
    tmp = tempfile.NamedTemporaryFile(delete=False)
    private_key, public_key = generate_pem_eccp256_keypair()
    tmp.write(private_key)
    tmp.close()
    yield tmp.name, public_key
    os.remove(tmp.name)


@pytest.fixture()
def tmp_file():
    tmp = tempfile.NamedTemporaryFile(delete=False)
    yield tmp
    tmp.close()
    os.remove(tmp.name)


@pytest.fixture(autouse=True)
@condition.capability(CAPABILITY.HSMAUTH)
@condition.min_version(5, 4, 3)
def preconditions(ckman_cli):
    ckman_cli("hsmauth", "reset", "-f")


class TestHsmAuth:
    def test_hsmauth_info(self, ckman_cli):
        output = ckman_cli("hsmauth", "info").output
        assert "version:" in output

    def test_hsmauth_reset(self, ckman_cli):
        output = ckman_cli("hsmauth", "reset", "-f").output
        assert (
            "Success! All YubiHSM Auth data have been cleared from the YubiKey."
            in output
        )


def calculate_session_keys_apdu(label, context, credential_password):
    data = (
        Tlv(TAG_LABEL, _parse_label(label))
        + Tlv(TAG_CONTEXT, context)
        + Tlv(TAG_CREDENTIAL_PASSWORD, _parse_credential_password(credential_password))
    )

    apdu = struct.pack("<BBBB", 0, INS_CALCULATE, 0, 0).hex()
    apdu = apdu + ":" + data.hex() + "=9000"

    return apdu


class TestCredentials:
    def verify_credential_password(self, ckman_cli, credential_password, label):
        context = b"g\xfc\xf1\xfe\xb5\xf1\xd8\x83\xedv=\xbfI0\x90\xbb"
        apdu = calculate_session_keys_apdu(label, context, credential_password)

        # Try to calculate session keys using credential password
        ckman_cli("apdu", "-a", "hsmauth", apdu)

    def test_import_credential_symmetric(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "credentials",
            "symmetric",
            "test-name-sym",
            "-c",
            "123456",
            "-E",
            os.urandom(16).hex(),
            "-M",
            os.urandom(16).hex(),
            "-m",
            DEFAULT_MANAGEMENT_KEY,
        )
        self.verify_credential_password(ckman_cli, "123456", "test-name-sym")
        creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "test-name-sym" in creds

    def test_import_credential_symmetric_generate(self, ckman_cli):
        output = ckman_cli(
            "hsmauth",
            "credentials",
            "symmetric",
            "test-name-sym-gen",
            "-c",
            "123456",
            "-g",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
        ).output
        self.verify_credential_password(ckman_cli, "123456", "test-name-sym-gen")
        assert "Generated ENC and MAC keys" in output

    def test_import_credential_symmetric_derived(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "credentials",
            "derive",
            "test-name-sym-derived",
            "-c",
            "123456",
            "-d",
            "password",
        )
        self.verify_credential_password(ckman_cli, "123456", "test-name-sym-derived")
        creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "test-name-sym-derived" in creds

    @condition.min_version(5, 6)
    def test_import_credential_asymmetric(self, ckman_cli):
        pair = generate_pem_eccp256_keypair()
        ckman_cli(
            "hsmauth",
            "credentials",
            "import",
            "test-name-asym",
            "-c",
            "123456",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            "-",
            input=pair[0],
        )
        creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "test-name-asym" in creds

        public_key_exported = ckman_cli(
            "hsmauth", "credentials", "export", "test-name-asym", "-"
        ).stdout_bytes
        assert pair[1] == public_key_exported

    @condition.min_version(5, 6)
    def test_generate_credential_asymmetric(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "credentials",
            "generate",
            "test-name-asym-generated",
            "-c",
            "123456",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
        )

        creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "test-name-asym-generated" in creds

    def test_import_credential_touch_required(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "credentials",
            "derive",
            "test-name-touch",
            "-c",
            "123456",
            "-d",
            "password",
            "-t",
        )

        creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "On" in creds
        assert "test-name-touch" in creds

    @condition.min_version(5, 6)
    def test_export_public_key_to_file(self, ckman_cli, eccp256_keypair, tmp_file):
        private_key_file, public_key = eccp256_keypair
        ckman_cli(
            "hsmauth",
            "credentials",
            "import",
            "test-name-asym",
            "-c",
            "123456",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            private_key_file,
        )

        ckman_cli(
            "hsmauth",
            "credentials",
            "export",
            "test-name-asym",
            tmp_file.name,
        )

        public_key_from_file = tmp_file.read()
        assert public_key_from_file == public_key

    @condition.min_version(5, 6)
    def test_export_public_key_symmetric_credential(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "credentials",
            "derive",
            "test-name-sym",
            "-c",
            "123456",
            "-d",
            "password",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
        )

        with pytest.raises(SystemExit):
            ckman_cli("hsmauth", "credentials", "export", "test-name-sym")

    def test_delete_credential(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "credentials",
            "derive",
            "delete-me",
            "-c",
            "123456",
            "-d",
            "password",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
        )
        old_creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "delete-me" in old_creds
        ckman_cli("hsmauth", "credentials", "delete", "delete-me", "-f")
        new_creds = ckman_cli("hsmauth", "credentials", "list").output
        assert "delete-me" not in new_creds


class TestManagementKey:
    def test_change_management_key(self, ckman_cli):
        ckman_cli(
            "hsmauth",
            "access",
            "change-management-key",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            "-n",
            NON_DEFAULT_MANAGEMENT_KEY,
        )

        with pytest.raises(SystemExit):
            # Should fail - wrong current key
            ckman_cli(
                "hsmauth",
                "access",
                "change-management-key",
                "-m",
                DEFAULT_MANAGEMENT_KEY,
                "-n",
                DEFAULT_MANAGEMENT_KEY,
            )

        # Should succeed
        ckman_cli(
            "hsmauth",
            "access",
            "change-management-key",
            "-m",
            NON_DEFAULT_MANAGEMENT_KEY,
            "-n",
            DEFAULT_MANAGEMENT_KEY,
        )

    def test_change_management_key_generate(self, ckman_cli):
        output = ckman_cli(
            "hsmauth",
            "access",
            "change-management-key",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            "-g",
        ).output

        assert re.match(
            r"^Generated management key: [a-f0-9]{16}", output, re.MULTILINE
        )



#  vim: set fileencoding=utf-8 :

from ckman import __version__ as version
from canokit.core import Tlv, bytes2int, InvalidPinError
from canokit.core.otp import modhex_encode, modhex_decode
from canokit.management import FORM_FACTOR
from ckman.util import is_pkcs12, is_pem, parse_private_key, parse_certificates
from ckman.util import _parse_pkcs12
from ckman.otp import format_oath_code, generate_static_pw, time_challenge
from .util import open_file

import unittest


def test_invalid_pin_exception_value_error():
    # Fail if InvalidPinError still inherits ValueError in ckman 6.0
    if int(version.split(".")[0]) != 5:
        assert not isinstance(InvalidPinError(3), ValueError)


class TestUtilityFunctions(unittest.TestCase):
    def test_bytes2int(self):
        self.assertEqual(0x57, bytes2int(b"\x57"))
        self.assertEqual(0x1234, bytes2int(b"\x12\x34"))
        self.assertEqual(0xCAFED00D, bytes2int(b"\xca\xfe\xd0\x0d"))

    def test_format_oath_code(self):
        self.assertEqual("000000", format_oath_code(b"\0" * 20))
        self.assertEqual("00000000", format_oath_code(b"\0" * 20, 8))
        self.assertEqual("345678", format_oath_code(b"\x00\xbc\x61\x4e" + b"\0" * 16))
        self.assertEqual(
            "34567890", format_oath_code(b"\x49\x96\x02\xd2" + b"\0" * 16, 8)
        )

    def test_generate_static_pw(self):
        for i in range(0, 38):
            self.assertRegex(
                generate_static_pw(i), "^[cbdefghijklnrtuvCBDEFGHIJKLNRTUV]{%d}$" % i
            )

    def test_modhex_decode(self):
        self.assertEqual(b"", modhex_decode(""))
        self.assertEqual(b"\x2d\x34\x4e\x83", modhex_decode("dteffuje"))
        self.assertEqual(
            b"\x69\xb6\x48\x1c\x8b\xab\xa2\xb6\x0e\x8f\x22\x17\x9b\x58\xcd\x56",
            modhex_decode("hknhfjbrjnlnldnhcujvddbikngjrtgh"),
        )

    def test_modhex_encode(self):
        self.assertEqual("", modhex_encode(b""))
        self.assertEqual("dteffuje", modhex_encode(b"\x2d\x34\x4e\x83"))
        self.assertEqual(
            "hknhfjbrjnlnldnhcujvddbikngjrtgh",
            modhex_encode(
                b"\x69\xb6\x48\x1c\x8b\xab\xa2\xb6" b"\x0e\x8f\x22\x17\x9b\x58\xcd\x56"
            ),
        )

    def test_parse_tlvs(self):
        tlvs = Tlv.parse_list(b"\x00\x02\xd0\x0d\xa1\x00\xfe\x04\xfe\xed\xfa\xce")
        self.assertEqual(3, len(tlvs))

        self.assertEqual(0, tlvs[0].tag)
        self.assertEqual(2, tlvs[0].length)
        self.assertEqual(b"\xd0\x0d", tlvs[0].value)

        self.assertEqual(0xA1, tlvs[1].tag)
        self.assertEqual(0, tlvs[1].length)
        self.assertEqual(b"", tlvs[1].value)

        self.assertEqual(0xFE, tlvs[2].tag)
        self.assertEqual(4, tlvs[2].length)
        self.assertEqual(b"\xfe\xed\xfa\xce", tlvs[2].value)

    def test_time_challenge(self):
        self.assertEqual(b"\0" * 8, time_challenge(0))
        self.assertEqual(b"\x00\x00\x00\x00\x00\x06G\x82", time_challenge(12345678))
        self.assertEqual(
            b"\x00\x00\x00\x00\x02\xf2\xeaC",
            time_challenge(1484223461.2644958),  # type: ignore
        )

    def test_tlv(self):
        self.assertEqual(Tlv(b"\xfe\6foobar"), Tlv(0xFE, b"foobar"))

        tlv1 = Tlv(b"\0\5hello")
        tlv2 = Tlv(0xFE, b"")
        tlv3 = Tlv(0x12, b"hi" * 200)

        self.assertEqual(b"\0\5hello", tlv1)
        self.assertEqual(b"\xfe\0", tlv2)
        self.assertEqual(b"\x12\x82\x01\x90" + b"hi" * 200, tlv3)

        self.assertEqual(
            b"\0\5hello\xfe\0\x12\x82\x01\x90" + b"hi" * 200, tlv1 + tlv2 + tlv3
        )

    def test_is_pkcs12(self):
        with self.assertRaises(TypeError):
            is_pkcs12(None)

        with open_file("rsa_2048_key.pem") as rsa_2048_key_pem:
            self.assertFalse(is_pkcs12(rsa_2048_key_pem.read()))

        with open_file("rsa_2048_key_encrypted.pem") as f:
            self.assertFalse(is_pkcs12(f.read()))

        with open_file("rsa_2048_cert.pem") as rsa_2048_cert_pem:
            self.assertFalse(is_pkcs12(rsa_2048_cert_pem.read()))

        with open_file("rsa_2048_key_cert.pfx") as rsa_2048_key_cert_pfx:
            data = rsa_2048_key_cert_pfx.read()
        self.assertTrue(is_pkcs12(data))
        parse_private_key(data, None)
        parse_certificates(data, None)

        with open_file(
            "rsa_2048_key_cert_encrypted.pfx"
        ) as rsa_2048_key_cert_encrypted_pfx:
            self.assertTrue(is_pkcs12(rsa_2048_key_cert_encrypted_pfx.read()))

    def test_parse_pkcs12(self):
        with open_file("rsa_2048_key_cert.pfx") as rsa_2048_key_cert_pfx:
            data = rsa_2048_key_cert_pfx.read()

        key, certs = _parse_pkcs12(data, None)
        self.assertEqual(1, len(certs))

    def test_is_pem(self):
        self.assertFalse(is_pem(b"just a byte string"))
        self.assertFalse(is_pem(None))

        with open_file("rsa_2048_key.pem") as rsa_2048_key_pem:
            self.assertTrue(is_pem(rsa_2048_key_pem.read()))

        with open_file("rsa_2048_key_encrypted.pem") as f:
            self.assertTrue(is_pem(f.read()))

        with open_file("rsa_2048_cert.pem") as rsa_2048_cert_pem:
            self.assertTrue(is_pem(rsa_2048_cert_pem.read()))

        with open_file("rsa_2048_key_cert.pfx") as rsa_2048_key_cert_pfx:
            self.assertFalse(is_pem(rsa_2048_key_cert_pfx.read()))

        with open_file("rsa_2048_cert_metadata.pem") as f:
            self.assertTrue(is_pem(f.read()))

        with open_file(
            "rsa_2048_key_cert_encrypted.pfx"
        ) as rsa_2048_key_cert_encrypted_pfx:
            self.assertFalse(is_pem(rsa_2048_key_cert_encrypted_pfx.read()))

    def test_form_factor_from_code(self):
        with self.assertRaises(ValueError):
            FORM_FACTOR.from_code("im a string")  # type: ignore
        self.assertEqual(FORM_FACTOR.UNKNOWN, FORM_FACTOR.from_code(0x00))
        self.assertEqual(FORM_FACTOR.USB_A_KEYCHAIN, FORM_FACTOR.from_code(0x01))
        self.assertEqual(FORM_FACTOR.USB_A_NANO, FORM_FACTOR.from_code(0x02))
        self.assertEqual(FORM_FACTOR.USB_C_KEYCHAIN, FORM_FACTOR.from_code(0x03))
        self.assertEqual(FORM_FACTOR.USB_C_NANO, FORM_FACTOR.from_code(0x04))
        self.assertEqual(FORM_FACTOR.USB_C_LIGHTNING, FORM_FACTOR.from_code(0x05))
        self.assertEqual(FORM_FACTOR.UNKNOWN, FORM_FACTOR.from_code(0x99))



from canokit.piv import OBJECT_ID
import pytest


DEFAULT_MANAGEMENT_KEY = "010203040506070801020304050607080102030405060708"


class TestMisc:
    def setUp(self, ckman_cli):
        ckman_cli("piv", "reset", "-f")

    def test_info(self, ckman_cli):
        output = ckman_cli("piv", "info").output
        assert "PIV version:" in output

    def test_reset(self, ckman_cli):
        output = ckman_cli("piv", "reset", "-f").output
        assert "Success!" in output

    def test_export_invalid_certificate_fails(self, ckman_cli):
        ckman_cli(
            "piv",
            "objects",
            "import",
            hex(OBJECT_ID.AUTHENTICATION),
            "-",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            input="This is not a cert",
        )

        with pytest.raises(SystemExit):
            ckman_cli(
                "piv", "certificates", "export", hex(OBJECT_ID.AUTHENTICATION), "-"
            )

    def test_info_with_invalid_certificate_does_not_crash(self, ckman_cli):
        ckman_cli(
            "piv",
            "objects",
            "import",
            hex(OBJECT_ID.AUTHENTICATION),
            "-",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            input="This is not a cert",
        )
        ckman_cli("piv", "info")



from canokit.management import CAPABILITY
from .. import condition
import pytest

DEFAULT_PIN = "123456"
NON_DEFAULT_PIN = "654321"
DEFAULT_ADMIN_PIN = "12345678"
NON_DEFAULT_ADMIN_PIN = "87654321"


def old_new_new(old, new):
    return f"{old}\n{new}\n{new}\n"


@pytest.fixture(autouse=True)
@condition.capability(CAPABILITY.OPENPGP)
def preconditions(ckman_cli):
    ckman_cli("openpgp", "reset", "-f")


class TestOpenPGP:
    def test_openpgp_info(self, ckman_cli):
        output = ckman_cli("openpgp", "info").output
        assert "OpenPGP version:" in output

    def test_openpgp_reset(self, ckman_cli):
        output = ckman_cli("openpgp", "reset", "-f").output
        assert "Success! All data has been cleared and default PINs are set." in output


class TestPin:
    def test_change_pin(self, ckman_cli):
        ckman_cli(
            "openpgp", "access", "change-pin", "-P", DEFAULT_PIN, "-n", NON_DEFAULT_PIN
        )
        ckman_cli(
            "openpgp", "access", "change-pin", "-P", NON_DEFAULT_PIN, "-n", DEFAULT_PIN
        )

    def test_change_pin_prompt(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "change-pin",
            input=old_new_new(DEFAULT_PIN, NON_DEFAULT_PIN),
        )
        ckman_cli(
            "openpgp",
            "access",
            "change-pin",
            input=old_new_new(NON_DEFAULT_PIN, DEFAULT_PIN),
        )


class TestAdminPin:
    def test_change_admin_pin(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "change-admin-pin",
            "-a",
            DEFAULT_ADMIN_PIN,
            "-n",
            NON_DEFAULT_ADMIN_PIN,
        )
        ckman_cli(
            "openpgp",
            "access",
            "change-admin-pin",
            "-a",
            NON_DEFAULT_ADMIN_PIN,
            "-n",
            DEFAULT_ADMIN_PIN,
        )

    def test_change_pin_prompt(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "change-admin-pin",
            input=old_new_new(DEFAULT_ADMIN_PIN, NON_DEFAULT_ADMIN_PIN),
        )
        ckman_cli(
            "openpgp",
            "access",
            "change-admin-pin",
            input=old_new_new(NON_DEFAULT_ADMIN_PIN, DEFAULT_ADMIN_PIN),
        )


class TestResetPin:
    def ensure_pin_changed(self, ckman_cli):
        ckman_cli(
            "openpgp", "access", "change-pin", "-P", NON_DEFAULT_PIN, "-n", DEFAULT_PIN
        )

    def test_set_and_use_reset_code(self, ckman_cli):
        reset_code = "12345678"

        ckman_cli(
            "openpgp",
            "access",
            "change-reset-code",
            "-a",
            DEFAULT_ADMIN_PIN,
            "-r",
            reset_code,
        )

        ckman_cli(
            "openpgp",
            "access",
            "unblock-pin",
            "-r",
            reset_code,
            "-n",
            NON_DEFAULT_PIN,
        )

        self.ensure_pin_changed(ckman_cli)

    def test_set_and_use_reset_code_prompt(self, ckman_cli):
        reset_code = "87654321"

        ckman_cli(
            "openpgp",
            "access",
            "change-reset-code",
            input=old_new_new(DEFAULT_ADMIN_PIN, reset_code),
        )

        ckman_cli(
            "openpgp",
            "access",
            "unblock-pin",
            input=old_new_new(reset_code, NON_DEFAULT_PIN),
        )

        ckman_cli(
            "openpgp", "access", "change-pin", "-P", NON_DEFAULT_PIN, "-n", DEFAULT_PIN
        )

    def test_unblock_pin_with_admin_pin(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "unblock-pin",
            "-a",
            DEFAULT_ADMIN_PIN,
            "-n",
            NON_DEFAULT_PIN,
        )

        self.ensure_pin_changed(ckman_cli)

    def test_unblock_pin_with_admin_pin_prompt(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "unblock-pin",
            "--admin-pin",
            "-",
            input=old_new_new(DEFAULT_ADMIN_PIN, NON_DEFAULT_PIN),
        )

        self.ensure_pin_changed(ckman_cli)


class TestForceSignature:
    def test_set_force_sig(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "set-signature-policy",
            "ALWAYS",
            "-a",
            DEFAULT_ADMIN_PIN,
        )

        output = ckman_cli("openpgp", "info").output
        assert "Always" in output

        ckman_cli(
            "openpgp", "access", "set-signature-policy", "ONCE", "-a", DEFAULT_ADMIN_PIN
        )

        output = ckman_cli("openpgp", "info").output
        assert "Once" in output

    def test_set_force_sig_prompt(self, ckman_cli):
        ckman_cli(
            "openpgp",
            "access",
            "set-signature-policy",
            "ALWAYS",
            input=DEFAULT_ADMIN_PIN,
        )

        output = ckman_cli("openpgp", "info").output
        assert "Always" in output

        ckman_cli(
            "openpgp",
            "access",
            "set-signature-policy",
            "ONCE",
            input=DEFAULT_ADMIN_PIN,
        )

        output = ckman_cli("openpgp", "info").output
        assert "Once" in output



from canokit.core import TRANSPORT, YUBIKEY
from canokit.management import (
    CAPABILITY,
    FORM_FACTOR,
    DeviceInfo,
    DeviceConfig,
    Version,
)
from canokit.support import get_name
from typing import cast


def info(form_factor):
    return DeviceInfo(
        config=cast(DeviceConfig, None),
        serial=None,
        version=Version(5, 3, 0),
        form_factor=form_factor,
        supported_capabilities={TRANSPORT.USB: 0xFF},  # type: ignore
        is_locked=False,
        is_fips=False,
    )


def info_nfc(form_factor):
    with_nfc = info(form_factor)
    with_nfc.supported_capabilities[TRANSPORT.NFC] = 0xFF
    return with_nfc


def test_yk5_formfactors():
    kt = YUBIKEY.YK4
    assert get_name(info(FORM_FACTOR.USB_A_KEYCHAIN), kt) == "YubiKey 5A"
    assert get_name(info_nfc(FORM_FACTOR.USB_A_KEYCHAIN), kt) == "YubiKey 5 NFC"
    assert get_name(info(FORM_FACTOR.USB_A_NANO), kt) == "YubiKey 5 Nano"
    assert get_name(info(FORM_FACTOR.USB_C_KEYCHAIN), kt) == "YubiKey 5C"
    assert get_name(info_nfc(FORM_FACTOR.USB_C_KEYCHAIN), kt) == "YubiKey 5C NFC"
    assert get_name(info(FORM_FACTOR.USB_C_NANO), kt) == "YubiKey 5C Nano"
    assert get_name(info(FORM_FACTOR.USB_C_LIGHTNING), kt) == "YubiKey 5Ci"
    assert get_name(info(FORM_FACTOR.USB_A_BIO), kt) == "YubiKey Bio"
    assert get_name(info(FORM_FACTOR.USB_C_BIO), kt) == "YubiKey C Bio"
    assert get_name(info(FORM_FACTOR.UNKNOWN), kt) == "YubiKey 5"
    assert get_name(info_nfc(FORM_FACTOR.UNKNOWN), kt) == "YubiKey 5 NFC"


def fido(device_info):
    device_info.supported_capabilities[TRANSPORT.USB] = (
        CAPABILITY.U2F | CAPABILITY.FIDO2
    )
    if TRANSPORT.NFC in device_info.supported_capabilities:
        device_info.supported_capabilities[TRANSPORT.NFC] = (
            CAPABILITY.U2F | CAPABILITY.FIDO2
        )
    return device_info


def test_yk5_fido():
    kt = YUBIKEY.YK4
    assert (
        get_name(fido(info(FORM_FACTOR.USB_A_BIO)), kt) == "YubiKey Bio - FIDO Edition"
    )
    assert (
        get_name(fido(info(FORM_FACTOR.USB_C_BIO)), kt)
        == "YubiKey C Bio - FIDO Edition"
    )


def fips(device_info):
    device_info.is_fips = True
    return device_info


def test_yk5_fips_formfactors():
    kt = YUBIKEY.YK4
    assert get_name(fips(info(FORM_FACTOR.USB_A_KEYCHAIN)), kt) == "YubiKey 5A FIPS"
    assert (
        get_name(fips(info_nfc(FORM_FACTOR.USB_A_KEYCHAIN)), kt) == "YubiKey 5 NFC FIPS"
    )
    assert get_name(fips(info(FORM_FACTOR.USB_A_NANO)), kt) == "YubiKey 5 Nano FIPS"
    assert get_name(fips(info(FORM_FACTOR.USB_C_KEYCHAIN)), kt) == "YubiKey 5C FIPS"
    assert (
        get_name(fips(info_nfc(FORM_FACTOR.USB_C_KEYCHAIN)), kt)
        == "YubiKey 5C NFC FIPS"
    )
    assert get_name(fips(info(FORM_FACTOR.USB_C_NANO)), kt) == "YubiKey 5C Nano FIPS"
    assert get_name(fips(info(FORM_FACTOR.USB_C_LIGHTNING)), kt) == "YubiKey 5Ci FIPS"
    assert get_name(fips(info(FORM_FACTOR.USB_A_BIO)), kt) == "YubiKey Bio FIPS"
    assert get_name(fips(info(FORM_FACTOR.USB_C_BIO)), kt) == "YubiKey C Bio FIPS"
    assert get_name(fips(info(FORM_FACTOR.UNKNOWN)), kt) == "YubiKey 5 FIPS"
    assert get_name(fips(info_nfc(FORM_FACTOR.UNKNOWN)), kt) == "YubiKey 5 NFC FIPS"


def sky(device_info):
    device_info.is_sky = True
    return device_info


def test_sky_formfactors():
    kt = YUBIKEY.YK4
    assert get_name(sky(info(FORM_FACTOR.USB_A_KEYCHAIN)), kt) == "Security Key A"
    assert get_name(sky(info_nfc(FORM_FACTOR.USB_A_KEYCHAIN)), kt) == "Security Key NFC"
    assert get_name(sky(info(FORM_FACTOR.USB_A_NANO)), kt) == "Security Key Nano"
    assert get_name(sky(info(FORM_FACTOR.USB_C_KEYCHAIN)), kt) == "Security Key C"
    assert (
        get_name(sky(info_nfc(FORM_FACTOR.USB_C_KEYCHAIN)), kt) == "Security Key C NFC"
    )
    assert get_name(sky(info(FORM_FACTOR.USB_C_NANO)), kt) == "Security Key C Nano"
    assert get_name(sky(info(FORM_FACTOR.USB_C_LIGHTNING)), kt) == "Security Key Ci"
    assert get_name(sky(info(FORM_FACTOR.UNKNOWN)), kt) == "Security Key"
    assert get_name(sky(info_nfc(FORM_FACTOR.UNKNOWN)), kt) == "Security Key NFC"


def skyep(device_info):
    device_info.is_sky = True
    device_info.serial = 123456
    return device_info


def test_sky_enterprise_formfactors():
    kt = YUBIKEY.YK4
    assert (
        get_name(skyep(info(FORM_FACTOR.USB_A_KEYCHAIN)), kt)
        == "Security Key A - Enterprise Edition"
    )
    assert (
        get_name(skyep(info_nfc(FORM_FACTOR.USB_A_KEYCHAIN)), kt)
        == "Security Key NFC - Enterprise Edition"
    )
    assert (
        get_name(skyep(info(FORM_FACTOR.USB_A_NANO)), kt)
        == "Security Key Nano - Enterprise Edition"
    )
    assert (
        get_name(skyep(info(FORM_FACTOR.USB_C_KEYCHAIN)), kt)
        == "Security Key C - Enterprise Edition"
    )
    assert (
        get_name(skyep(info_nfc(FORM_FACTOR.USB_C_KEYCHAIN)), kt)
        == "Security Key C NFC - Enterprise Edition"
    )
    assert (
        get_name(skyep(info(FORM_FACTOR.USB_C_NANO)), kt)
        == "Security Key C Nano - Enterprise Edition"
    )
    assert (
        get_name(skyep(info(FORM_FACTOR.USB_C_LIGHTNING)), kt)
        == "Security Key Ci - Enterprise Edition"
    )
    assert (
        get_name(skyep(info(FORM_FACTOR.UNKNOWN)), kt)
        == "Security Key - Enterprise Edition"
    )
    assert (
        get_name(skyep(info_nfc(FORM_FACTOR.UNKNOWN)), kt)
        == "Security Key NFC - Enterprise Edition"
    )



import pytest
import contextlib
import io


def test_fail_on_alias(ckman_cli):
    with io.StringIO() as buf:
        with contextlib.redirect_stderr(buf):
            with pytest.raises(SystemExit):
                ckman_cli("oath", "code")
        err = buf.getvalue()

    assert "oath accounts code" in err



from canokit.core import NotSupportedError
from ....util import open_file
from ... import condition
import pytest


class TestFIPS:
    @condition.yk4_fips(True)
    def test_rsa1024_generate_blocked(self, ckman_cli):
        with pytest.raises(NotSupportedError):
            ckman_cli("piv", "keys", "generate", "9a", "-a", "RSA1024", "-")

    @condition.yk4_fips(True)
    def test_rsa1024_import_blocked(self, ckman_cli):
        with pytest.raises(NotSupportedError):
            with open_file("rsa_1024_key.pem") as f:
                ckman_cli("piv", "keys", "import", "9a", f.name)



from .util import (
    old_new_new,
    DEFAULT_PIN,
    NON_DEFAULT_PIN,
    DEFAULT_PUK,
    NON_DEFAULT_PUK,
    DEFAULT_MANAGEMENT_KEY,
)
from ckman.piv import OBJECT_ID_PIVMAN_DATA, PivmanData

import pytest
import re


class TestPin:
    def test_change_pin(self, ckman_cli):
        ckman_cli(
            "piv", "access", "change-pin", "-P", DEFAULT_PIN, "-n", NON_DEFAULT_PIN
        )
        ckman_cli(
            "piv", "access", "change-pin", "-P", NON_DEFAULT_PIN, "-n", DEFAULT_PIN
        )

    def test_change_pin_prompt(self, ckman_cli):
        ckman_cli(
            "piv",
            "access",
            "change-pin",
            input=old_new_new(DEFAULT_PIN, NON_DEFAULT_PIN),
        )
        ckman_cli(
            "piv",
            "access",
            "change-pin",
            input=old_new_new(NON_DEFAULT_PIN, DEFAULT_PIN),
        )


class TestPuk:
    def test_change_puk(self, ckman_cli):
        o1 = ckman_cli(
            "piv", "access", "change-puk", "-p", DEFAULT_PUK, "-n", NON_DEFAULT_PUK
        ).output
        assert "New PUK set." in o1

        o2 = ckman_cli(
            "piv", "access", "change-puk", "-p", NON_DEFAULT_PUK, "-n", DEFAULT_PUK
        ).output
        assert "New PUK set." in o2

        with pytest.raises(SystemExit):
            ckman_cli(
                "piv", "access", "change-puk", "-p", NON_DEFAULT_PUK, "-n", DEFAULT_PUK
            )

    def test_change_puk_prompt(self, ckman_cli):
        ckman_cli(
            "piv",
            "access",
            "change-puk",
            input=old_new_new(DEFAULT_PUK, NON_DEFAULT_PUK),
        )
        ckman_cli(
            "piv",
            "access",
            "change-puk",
            input=old_new_new(NON_DEFAULT_PUK, DEFAULT_PUK),
        )

    def test_unblock_pin(self, ckman_cli):
        for _ in range(3):
            with pytest.raises(SystemExit):
                ckman_cli(
                    "piv",
                    "access",
                    "change-pin",
                    "-P",
                    NON_DEFAULT_PIN,
                    "-n",
                    DEFAULT_PIN,
                )

        o = ckman_cli("piv", "info").output
        assert re.search(r"PIN tries remaining:\s+0(/3)?", o)

        with pytest.raises(SystemExit):
            ckman_cli(
                "piv", "access", "change-pin", "-p", DEFAULT_PIN, "-n", NON_DEFAULT_PIN
            )

        o = ckman_cli(
            "piv", "access", "unblock-pin", "-p", DEFAULT_PUK, "-n", DEFAULT_PIN
        ).output
        assert "PIN unblocked" in o
        o = ckman_cli("piv", "info").output
        assert re.search(r"PIN tries remaining:\s+3(/3)?", o)


class TestSetRetries:
    def test_set_retries(self, ckman_cli, version):
        ckman_cli(
            "piv",
            "access",
            "set-retries",
            "5",
            "6",
            input=f"{DEFAULT_MANAGEMENT_KEY}\n{DEFAULT_PIN}\ny\n",
        )

        o = ckman_cli("piv", "info").output
        assert re.search(r"PIN tries remaining:\s+5(/5)?", o)
        if version >= (5, 3):
            assert re.search(r"PUK tries remaining:\s+6/6", o)

    def test_set_retries_clears_puk_blocked(self, ckman_cli):
        for _ in range(3):
            with pytest.raises(SystemExit):
                ckman_cli(
                    "piv",
                    "access",
                    "change-puk",
                    "-p",
                    NON_DEFAULT_PUK,
                    "-n",
                    DEFAULT_PUK,
                )

        pivman = PivmanData()
        pivman.puk_blocked = True

        ckman_cli(
            "piv",
            "objects",
            "import",
            hex(OBJECT_ID_PIVMAN_DATA),
            "-",
            "-m",
            DEFAULT_MANAGEMENT_KEY,
            input=pivman.get_bytes(),
        )

        o = ckman_cli("piv", "info").output
        assert "PUK is blocked" in o

        ckman_cli(
            "piv",
            "access",
            "set-retries",
            "3",
            "3",
            input=f"{DEFAULT_MANAGEMENT_KEY}\n{DEFAULT_PIN}\ny\n",
        )

        o = ckman_cli("piv", "info").output
        assert "PUK is blocked" not in o



from ckman.piv import generate_random_management_key, parse_rfc4514_string

from canokit.core import NotSupportedError, Version
from canokit.piv import (
    KEY_TYPE,
    MANAGEMENT_KEY_TYPE,
    PIN_POLICY,
    TOUCH_POLICY,
    check_key_support,
)

import pytest


@pytest.mark.parametrize(
    "value",
    [
        r"UID=jsmith,DC=example,DC=net",
        r"OU=Sales+CN=J.  Smith,DC=example,DC=net",
        r"CN=James \"Jim\" Smith\, III,DC=example,DC=net",
        r"CN=Before\0dAfter,DC=example,DC=net",
        r"1.3.6.1.4.1.1466.0=#04024869",
        r"CN=Lu\C4\8Di\C4\87",
        r"1.2.840.113549.1.9.1=user@example.com",
    ],
)
def test_parse_rfc4514_string(value):
    name = parse_rfc4514_string(value)
    name2 = parse_rfc4514_string(name.rfc4514_string())
    assert name == name2


class TestPivFunctions:
    def test_generate_random_management_key(self):
        output1 = generate_random_management_key(MANAGEMENT_KEY_TYPE.TDES)
        output2 = generate_random_management_key(MANAGEMENT_KEY_TYPE.TDES)
        assert isinstance(output1, bytes)
        assert isinstance(output2, bytes)
        assert output1 != output2

        assert 24 == len(generate_random_management_key(MANAGEMENT_KEY_TYPE.TDES))

        assert 16 == len(generate_random_management_key(MANAGEMENT_KEY_TYPE.AES128))
        assert 24 == len(generate_random_management_key(MANAGEMENT_KEY_TYPE.AES192))
        assert 32 == len(generate_random_management_key(MANAGEMENT_KEY_TYPE.AES256))

    def test_supported_algorithms(self):
        with pytest.raises(NotSupportedError):
            check_key_support(
                Version(3, 1, 1),
                KEY_TYPE.ECCP384,
                PIN_POLICY.DEFAULT,
                TOUCH_POLICY.DEFAULT,
            )

        with pytest.raises(NotSupportedError):
            check_key_support(
                Version(4, 4, 1),
                KEY_TYPE.RSA1024,
                PIN_POLICY.DEFAULT,
                TOUCH_POLICY.DEFAULT,
            )

        for key_type in (KEY_TYPE.RSA1024, KEY_TYPE.RSA2048):
            with pytest.raises(NotSupportedError):
                check_key_support(
                    Version(4, 3, 4), key_type, PIN_POLICY.DEFAULT, TOUCH_POLICY.DEFAULT
                )

        for key_type in (KEY_TYPE.ED25519, KEY_TYPE.X25519):
            with pytest.raises(NotSupportedError):
                check_key_support(
                    Version(5, 6, 0), key_type, PIN_POLICY.DEFAULT, TOUCH_POLICY.DEFAULT
                )

        for key_type in KEY_TYPE:
            check_key_support(
                Version(5, 7, 0), key_type, PIN_POLICY.DEFAULT, TOUCH_POLICY.DEFAULT
            )
